QPalette {
    Base: transparent;
    Text: rgb(25,25,25);
    Window: rgb(255,255,255);
    WindowText: rgb(25,25,25);
    Highlight: rgba(44, 167, 248, 0.8);
    HighlightedText: rgba(255,255,255, 0.8);
    Button: qlineargradient(x1:0, y1:0 x2:0, y2:1, stop:0 "#fff", stop:1 "#fdfdfd");
    ButtonText: "#000";
}

QPalette:disabled {
    Text: rgba(0, 0, 0, 0.4);
    WindowText: rgba(0, 0, 0, 0.4);
    Highlight: rgba(44, 167, 248, 0.4);
    HighlightedText: rgba(255, 255, 255, 0.4);
}

Slider {
    GrooveColor: "#a4a4a4";
    GrooveHighlightColor: "#2ca7f8";
    HandleBrush: url(:/assets/semilight/slider/slider_handle_normal.svg);
    VernierHandleBrush: url(:/assets/semilight/slider/slider_point_handle.png);
    HandleBorderColor: lightgray;
    TickmarkColor: rgba(0,0,0,0.6);
}

Slider:focus {
    HandleBrush: url(:/assets/semilight/slider/slider_handle_focus.svg);
    VernierHandleBrush: url(:/assets/semilight/slider/slider_point_handle_focus.png);
    HandleBorderColor: "#2ca7f8";
}

Slider:disabled {
    GrooveColor: rgba(163, 163, 163, 0.4);
    GrooveHighlightColor: "#aedfff";
    HandleBrush: url(:/assets/semilight/slider/slider_handle_insensitive.svg);
    VernierHandleBrush: url(:/assets/semilight/slider/slider_point_handle_insensitive.png);
    HandleBorderColor: rgba(0, 0, 0, 0.1);
}

Slider:pressed {
    HandleBrush: url(:/assets/semilight/slider/slider_handle_press.svg);
    VernierHandleBrush: url(:/assets/semilight/slider/slider_point_handle_press.png);
}

ProgressBar {
    GrooveBackground: rgba(0, 0, 0, 0.05);
    GrooveBorder: rgba(255, 255, 255, 0.05);
    ContentBackground: "#2ca7f8";
    ContentBorder: "#378cfa";
}

PushButton {
    BackgroundBrush: qlineargradient(x1:0, y1:0 x2:0, y2:1, stop:0 "#fff", stop:1 "#fdfdfd");
    TextColor: "#000";
    SuggestTextColor: rgba(0, 132, 255, 1);
    BorderBrush: rgba(0,0,0,0.1);
    SuggestButtonColor: rgba(0, 132, 255, 0.4);
}

PushButton:disabled {
    BackgroundBrush: qlineargradient(x1: 0, y1: 0 x2: 0, y2: 1, stop:0 rgba(255, 255, 255, 0.4), stop:1 rgba(253, 253, 253, 0.4));
    TextColor: "#AEAEAE";
    SuggestTextColor: "#AEAEAE";
    BorderBrush: rgba(0,0,0,0.04);
    SuggestButtonColor: rgba(0,0,0,0.04);
}

PushButton:hover {
    BackgroundBrush: qlineargradient(x1: 0, y1: 0 x2: 0, y2: 1, stop:0 "#8ccfff", stop:1 "#4bb8ff");
    TextColor: "#fff";
    SuggestTextColor: "#fff";
    BorderBrush: rgba(0, 117, 243, 0.2);
    SuggestButtonColor: rgba(0, 117, 243, 0.2);
}

PushButton:pressed {
    BackgroundBrush: qlineargradient(x1: 0, y1: 0 x2: 0, y2: 1, stop:0 "#0b8cff", stop:1 "#0aa1ff");
    TextColor: "#fff";
    SuggestTextColor: "#fff";
    BorderBrush: rgba(0, 0, 0, 0.05);
    SuggestButtonColor: rgba(29, 129, 255, 0.3);
}

PushButton:focus {
    BorderBrush: "#2ca7f8";
    SuggestButtonColor: rgba(0, 132, 255, 0.4);
}

PushButton:flat {
    TextColor: "#0082fa";
    SuggestTextColor: "#0082fa";
}

PushButton:flat:disabled {
    TextColor: "#0060b9";
    SuggestTextColor: "#0060b9";
}

PushButton:flat:hover {
    TextColor: "#16b8ff";
    SuggestTextColor: "#16b8ff";
}

PushButton:flat:pressed {
    TextColor: "#0082fa";
    SuggestTextColor: "#0082fa";
}

LineEdit {
    BackgroundBrush: rgb(255,255,255);
    BorderBrush: rgba(0,0,0,0.1);
    AlertBorderBrush: "#FF8F00";
}

LineEdit:disabled {
    BackgroundBrush: rgb(255,255,255);
    BorderBrush: rgba(0,0,0,0.04);
}

LineEdit:focus {
    BackgroundBrush: rgb(255,255,255);
    BorderBrush: rgb(44,167,248);
}

ScrollBar {
    HandleBrush: rgba(16, 16, 16, 0.5);
}

ScrollBar:hover {
    HandleBrush: rgba(16, 16, 16, 0.5);
}

ScrollBar:pressed {
    HandleBrush: rgba(16, 16, 16, 0.55);
}

ScrollBar:focus {
    HandleBorderBrush: "#2ca7f8";
}

Menu {
    SeparatorColor: rgba(0, 0, 0, 0.1);
    BorderColor: rgba(0, 0, 0, 0.1);
}

CheckBox {
    BackgroundBrush: url(:/assets/light/checkbox/checkbox_unchecked.png);
}

CheckBox:disabled {
    BackgroundBrush: url(:/assets/light/checkbox/checkbox_unchecked_insensitive.png);
}

CheckBox:focus {
    BackgroundBrush: url(:/assets/light/checkbox/checkbox_unchecked_focus.png);
}

CheckBox:hover {
    BackgroundBrush: url(:/assets/light/checkbox/checkbox_unchecked_hover.png);
}

CheckBox:pressed {
    BackgroundBrush: url(:/assets/light/checkbox/checkbox_unchecked_press.png);
}

CheckBox:checked {
    BackgroundBrush: url(:/assets/light/checkbox/checkbox_checked.png);
}

CheckBox:checked:disabled {
    BackgroundBrush: url(:/assets/light/checkbox/checkbox_checked_insensitive.png);
}

CheckBox:checked:focus {
    BackgroundBrush: url(:/assets/light/checkbox/checkbox_checked_focus.png);
}

CheckBox:checked:hover {
    BackgroundBrush: url(:/assets/light/checkbox/checkbox_checked_hover.png);
}

CheckBox:checked:pressed {
    BackgroundBrush: url(:/assets/light/checkbox/checkbox_checked_press.png);
}

CheckBox:indeterminate {
    BackgroundBrush: url(:/assets/light/checkbox/checkbox_mixed.png);
}

CheckBox:indeterminate:disabled {
    BackgroundBrush: url(:/assets/light/checkbox/checkbox_mixed_insensitive.png);
}

CheckBox:indeterminate:focus {
    BackgroundBrush: url(:/assets/light/checkbox/checkbox_mixed_focus.png);
}

CheckBox:indeterminate:hover {
    BackgroundBrush: url(:/assets/light/checkbox/checkbox_mixed_hover.png);
}

CheckBox:indeterminate:pressed {
    BackgroundBrush: url(:/assets/light/checkbox/checkbox_mixed_press.png);
}

RadioButton {
    BackgroundBrush: url(:/assets/light/checkbox/radio_unchecked.png);
}

RadioButton:disabled {
    BackgroundBrush: url(:/assets/light/checkbox/radio_unchecked_insensitive.png);
}

RadioButton:focus {
    BackgroundBrush: url(:/assets/light/checkbox/radio_unchecked_focus.png);
}

RadioButton:hover {
    BackgroundBrush: url(:/assets/light/checkbox/radio_unchecked_hover.png);
}

RadioButton:pressed {
    BackgroundBrush: url(:/assets/light/checkbox/radio_unchecked_press.png);
}

RadioButton:checked {
    BackgroundBrush: url(:/assets/light/checkbox/radio_checked.png);
}

RadioButton:checked:disabled {
    BackgroundBrush: url(:/assets/light/checkbox/radio_checked_insensitive.png);
}

RadioButton:checked:focus {
    BackgroundBrush: url(:/assets/light/checkbox/radio_checked_focus.png);
}

RadioButton:checked:hover {
    BackgroundBrush: url(:/assets/light/checkbox/radio_checked_hover.png);
}

RadioButton:checked:pressed {
    BackgroundBrush: url(:/assets/light/checkbox/radio_checked_press.png);
}

RadioButton:indeterminate {
    BackgroundBrush: url(:/assets/light/checkbox/radio_mixed.png);
}

RadioButton:indeterminate:disabled {
    BackgroundBrush: url(:/assets/light/checkbox/radio_mixed_insensitive.png);
}

RadioButton:indeterminate:focus {
    BackgroundBrush: url(:/assets/light/checkbox/radio_mixed_focus.png);
}

RadioButton:indeterminate:hover {
    BackgroundBrush: url(:/assets/light/checkbox/radio_mixed_hover.png);
}

RadioButton:indeterminate:pressed {
    BackgroundBrush: url(:/assets/light/checkbox/radio_mixed_press.png);
}

RubberBand {
    BackgroundBrush: rgba(43, 167, 248, 0.3);
    BorderBrush: rgba(30, 126, 255, 0.2);
}

HeaderView {
    ArrowIndicatorUpIcon: url(:/assets/light/header_arrow_up.png);
    ArrowIndicatorDownIcon: url(:/assets/light/header_arrow_down.png);
}

SpinBox {
    MinusBrush: url(:/assets/light/spinbox/spinbutton_minus_normal.png);
    PlusBrush: url(:/assets/light/spinbox/spinbutton_plus_normal.png);
    DownBrush: url(:/assets/light/spinbox/spinbutton_down_normal.png);
    UpBrush: url(:/assets/light/spinbox/spinbutton_up_normal.png);
    BorderColor: rgba(0,0,0,0.1);
}

SpinBox:disabled {
    MinusBrush: url(:/assets/light/spinbox/spinbutton_minus_insensitive.png);
    PlusBrush: url(:/assets/light/spinbox/spinbutton_plus_insensitive.png);
    DownBrush: url(:/assets/light/spinbox/spinbutton_down_insensitive.png);
    UpBrush: url(:/assets/light/spinbox/spinbutton_up_insensitive.png);
    BorderColor: rgba(0,0,0,0.04);
}

SpinBox:focus {
    BorderColor: rgb(44,167,248);
}

SpinBox:pressed {
    MinusBrush: url(:/assets/light/spinbox/spinbutton_minus_press.png);
    PlusBrush: url(:/assets/light/spinbox/spinbutton_plus_press.png);
    DownBrush: url(:/assets/light/spinbox/spinbutton_down_press.png);
    UpBrush: url(:/assets/light/spinbox/spinbutton_up_press.png);
}

SpinBox:hover {
    MinusBrush: url(:/assets/light/spinbox/spinbutton_minus_hover.png);
    PlusBrush: url(:/assets/light/spinbox/spinbutton_plus_hover.png);
    DownBrush: url(:/assets/light/spinbox/spinbutton_down_hover.png);
    UpBrush: url(:/assets/light/spinbox/spinbutton_up_hover.png);
}

TabBarScrollButton {
    BackgroundBrush: transparent;
    UpIcon: url(:/assets/light/tabbar/pageup_normal.svg);
    DownIcon: url(:/assets/light/tabbar/pagedown_normal.svg);
}

TabBarScrollButton:disabled {
    UpIcon: url(:/assets/light/tabbar/pageup_insensitive.svg);
    DownIcon: url(:/assets/light/tabbar/pagedown_insensitive.svg);
}

TabBarScrollButton:pressed {
    UpIcon: url(:/assets/light/tabbar/pageup_press.svg);
    DownIcon: url(:/assets/light/tabbar/pagedown_press.svg);
}

TabBarScrollButton:hover {
    UpIcon: url(:/assets/light/tabbar/pageup_hover.svg);
    DownIcon: url(:/assets/light/tabbar/pagedown_hover.svg);
}

TabBarAddButton {
    BackgroundBrush: transparent;
    Icon: url(:/assets/light/tabbar/add_normal.svg);
}

TabBarAddButton:pressed {
    Icon: url(:/assets/light/tabbar/add_press.svg);
}

TabBarAddButton:hover {
    Icon: url(:/assets/light/tabbar/add_hover.svg);
}

TabBarTab {
    BackgroundBrush: transparent;
    BorderBrush: rgba(0,0,0,0.1);
    TextColor: black;
    ActiveColor: "#2ca7f8";
    CloseIcon: url(:/assets/light/tabbar/tab_close_normal.svg);
}

TabBarTab:disabled {
    TextColor: "#949494";
}

TabBarTab:pressed {
    CloseIcon: url(:/assets/light/tabbar/tab_close_press.svg);
}

TabBarTab:hover {
    BackgroundBrush: rgba(0, 0, 0, 0.05);
    CloseIcon: url(:/assets/light/tabbar/tab_close_hover.svg);
}
