QPalette {
    Base: transparent;
    Text: rgb(255,255,255);
    Window: "#202020";
    WindowText: rgb(255,255,255);
    Highlight: rgba(44, 167, 248, 0.8);
    HighlightedText: rgba(255,255,255, 0.8);
    Button: rgba(255, 255, 255, 0.15);
    ButtonText: "#dcdcdc";
}

QPalette:disabled {
    Text: rgba(255,255,255, 0.4);
    WindowText: rgba(255,255,255, 0.4);
    Highlight: rgba(44, 167, 248, 0.4);
    HighlightedText: rgba(255,255,255, 0.4);
}

Slider {
    GrooveColor: rgba(255, 255, 255, 0.2);
    GrooveHighlightColor: white;
    HandleBrush: url(:/assets/semidark/slider/slider_handle_normal.svg);
    VernierHandleBrush: url(:/assets/semidark/slider/slider_point_handle.png);
    HandleBorderColor: transparent;
    TickmarkColor: rgba(255,255,255,0.6);
}

Slider:focus {
    HandleBrush: url(:/assets/semidark/slider/slider_handle_focus.svg);
    VernierHandleBrush: url(:/assets/semidark/slider/slider_point_handle_focus.png);
    HandleBorderColor: "#2ca7f8";
}

Slider:disabled {
    GrooveColor: rgba(255, 255, 255, 0.08);
    GrooveHighlightColor: "#0d4265";
    HandleBrush: url(:/assets/semidark/slider/slider_handle_insensitive.svg);
    VernierHandleBrush: url(:/assets/semidark/slider/slider_point_handle_insensitive.png);
    HandleBorderColor: rgba(0, 0, 0, 0.1);
}

Slider:pressed {
    HandleBrush: url(:/assets/semidark/slider/slider_handle_press.svg);
    VernierHandleBrush: url(:/assets/semidark/slider/slider_point_handle_press.png);
}

ProgressBar {
    GrooveBackground: rgba(255, 255, 255, 0.1);
    GrooveBorder: transparent;
    ContentBackground: "#2ca7f8";
    ContentBorder: "#378cfa";
}

PushButton {
    BackgroundBrush: rgba(255, 255, 255, 0.15);
    TextColor: "white";
    SuggestTextColor: rgba(0, 132, 255, 1);
    BorderBrush: transparent;
    SuggestButtonColor: rgba(0, 132, 255, 0.4);
}

PushButton:disabled {
    BackgroundBrush: rgba(255, 255, 255, 0.06);
    TextColor: "#40dcdcdc";
    SuggestTextColor: "#40dcdcdc";
    BorderBrush: transparent;
    SuggestButtonColor: transparent;
}

PushButton:hover {
    BackgroundBrush: rgba(255, 255, 255, 0.3);
    TextColor: white;
    SuggestTextColor: white;
    BorderBrush: transparent;
    SuggestButtonColor: transparent;
}

PushButton:pressed {
    BackgroundBrush: rgba(255,255,255,0.1);
    TextColor: "#2ca7f8";
    SuggestTextColor: "#2ca7f8";
    BorderBrush: rgba(0, 0, 0, 0.05);
    SuggestButtonColor: transparent;
}

PushButton:focus {
    BorderBrush: "#2ca7f8";
    SuggestButtonColor: rgba(0, 132, 255, 0.4);
}

PushButton:flat {
    TextColor: "#0082fa";
    SuggestTextColor: "#0082fa";
}

PushButton:flat:disabled {
    TextColor: "#0060b9";
    SuggestTextColor: "#0060b9";
}

PushButton:flat:hover {
    TextColor: "#16b8ff";
    SuggestTextColor: "#16b8ff";
}

LineEdit {
    BackgroundBrush: rgba(255,255,255,0.15);
    BorderBrush: transparent;
    AlertBorderBrush: "#FF8F00";
}

LineEdit:disabled {
    BackgroundBrush: rgba(255,255,255,0.04);
}

LineEdit:focus {
    BackgroundBrush: rgba(255,255,255,0.05);
    BorderBrush: rgba(43,167,248,0.6);
}

ScrollBar {
    HandleBrush: rgba(255, 255, 255, 0.2);
    HandleBroderBrush: rgba(255, 255, 255, 0.05);
}

ScrollBar:hover {
    HandleBrush: rgba(255, 255, 255, 0.4);
    HandleBroderBrush: rgba(255, 255, 255, 0.1);
}

ScrollBar:pressed {
    HandleBrush: rgba(96, 96, 96, 0.3);
    HandleBorderBrush: rgba(255, 255, 255, 0.05);
}

ScrollBar:focus {
    HandleBroderBrush: "#2ca7f8";
}

Menu {
    BackgroundBrush: "#313131";
    SeparatorColor: rgba(255, 255, 255, 0.1);
    BorderColor: rgba(0, 0, 0, 0.1);
}

CheckBox {
    BackgroundBrush: url(:/assets/dark/checkbox/checkbox_unchecked.png);
}

CheckBox:disabled {
    BackgroundBrush: url(:/assets/dark/checkbox/checkbox_unchecked_insensitive.png);
}

CheckBox:focus {
    BackgroundBrush: url(:/assets/dark/checkbox/checkbox_unchecked_focus.png);
}

CheckBox:hover {
    BackgroundBrush: url(:/assets/dark/checkbox/checkbox_unchecked_hover.png);
}

CheckBox:pressed {
    BackgroundBrush: url(:/assets/dark/checkbox/checkbox_unchecked_press.png);
}

CheckBox:checked {
    BackgroundBrush: url(:/assets/dark/checkbox/checkbox_checked.png);
}

CheckBox:checked:disabled {
    BackgroundBrush: url(:/assets/dark/checkbox/checkbox_checked_insensitive.png);
}

CheckBox:checked:focus {
    BackgroundBrush: url(:/assets/dark/checkbox/checkbox_checked_focus.png);
}

CheckBox:checked:hover {
    BackgroundBrush: url(:/assets/dark/checkbox/checkbox_checked_hover.png);
}

CheckBox:checked:pressed {
    BackgroundBrush: url(:/assets/dark/checkbox/checkbox_checked_press.png);
}

CheckBox:indeterminate {
    BackgroundBrush: url(:/assets/dark/checkbox/checkbox_mixed.png);
}

CheckBox:indeterminate:disabled {
    BackgroundBrush: url(:/assets/dark/checkbox/checkbox_mixed_insensitive.png);
}

CheckBox:indeterminate:focus {
    BackgroundBrush: url(:/assets/dark/checkbox/checkbox_mixed_focus.png);
}

CheckBox:indeterminate:hover {
    BackgroundBrush: url(:/assets/dark/checkbox/checkbox_mixed_hover.png);
}

CheckBox:indeterminate:pressed {
    BackgroundBrush: url(:/assets/dark/checkbox/checkbox_mixed_press.png);
}

RadioButton {
    BackgroundBrush: url(:/assets/dark/checkbox/radio_unchecked.png);
}

RadioButton:disabled {
    BackgroundBrush: url(:/assets/dark/checkbox/radio_unchecked_insensitive.png);
}

RadioButton:focus {
    BackgroundBrush: url(:/assets/dark/checkbox/radio_unchecked_focus.png);
}

RadioButton:hover {
    BackgroundBrush: url(:/assets/dark/checkbox/radio_unchecked_hover.png);
}

RadioButton:pressed {
    BackgroundBrush: url(:/assets/dark/checkbox/radio_unchecked_press.png);
}

RadioButton:checked {
    BackgroundBrush: url(:/assets/dark/checkbox/radio_checked.png);
}

RadioButton:checked:disabled {
    BackgroundBrush: url(:/assets/dark/checkbox/radio_checked_insensitive.png);
}

RadioButton:checked:focus {
    BackgroundBrush: url(:/assets/dark/checkbox/radio_checked_focus.png);
}

RadioButton:checked:hover {
    BackgroundBrush: url(:/assets/dark/checkbox/radio_checked_hover.png);
}

RadioButton:checked:pressed {
    BackgroundBrush: url(:/assets/dark/checkbox/radio_checked_press.png);
}

RadioButton:indeterminate {
    BackgroundBrush: url(:/assets/dark/checkbox/radio_mixed.png);
}

RadioButton:indeterminate:disabled {
    BackgroundBrush: url(:/assets/dark/checkbox/radio_mixed_insensitive.png);
}

RadioButton:indeterminate:focus {
    BackgroundBrush: url(:/assets/dark/checkbox/radio_mixed_focus.png);
}

RadioButton:indeterminate:hover {
    BackgroundBrush: url(:/assets/dark/checkbox/radio_mixed_hover.png);
}

RadioButton:indeterminate:pressed {
    BackgroundBrush: url(:/assets/dark/checkbox/radio_mixed_press.png);
}

RubberBand {
    BackgroundBrush: rgba(43, 167, 248, 0.3);
    BorderBrush: rgba(30, 126, 255, 0.2);
}

HeaderView {
    ArrowIndicatorUpIcon: url(:/assets/dark/header_arrow_up.png);
    ArrowIndicatorDownIcon: url(:/assets/dark/header_arrow_down.png);
}

SpinBox {
    MinusBrush: url(:/assets/dark/spinbox/spinbutton_minus_normal.png);
    PlusBrush: url(:/assets/dark/spinbox/spinbutton_plus_normal.png);
    DownBrush: url(:/assets/dark/spinbox/spinbutton_down_normal.png);
    UpBrush: url(:/assets/dark/spinbox/spinbutton_up_normal.png);
    BorderColor: black;
}

SpinBox:disabled {
    MinusBrush: url(:/assets/dark/spinbox/spinbutton_minus_insensitive.png);
    PlusBrush: url(:/assets/dark/spinbox/spinbutton_plus_insensitive.png);
    DownBrush: url(:/assets/dark/spinbox/spinbutton_down_insensitive.png);
    UpBrush: url(:/assets/dark/spinbox/spinbutton_up_insensitive.png);
    BorderColor: black;
}

SpinBox:focus {
    BorderColor: rgba(43,167,248,0.6);
}

SpinBox:pressed {
    MinusBrush: url(:/assets/dark/spinbox/spinbutton_minus_press.png);
    PlusBrush: url(:/assets/dark/spinbox/spinbutton_plus_press.png);
    DownBrush: url(:/assets/dark/spinbox/spinbutton_down_press.png);
    UpBrush: url(:/assets/dark/spinbox/spinbutton_up_press.png);
}

SpinBox:hover {
    MinusBrush: url(:/assets/dark/spinbox/spinbutton_minus_hover.png);
    PlusBrush: url(:/assets/dark/spinbox/spinbutton_plus_hover.png);
    DownBrush: url(:/assets/dark/spinbox/spinbutton_down_hover.png);
    UpBrush: url(:/assets/dark/spinbox/spinbutton_up_hover.png);
}

TabBarScrollButton {
    BackgroundBrush: transparent;
    UpIcon: url(:/assets/dark/tabbar/pageup_normal.svg);
    DownIcon: url(:/assets/dark/tabbar/pagedown_normal.svg);
}

TabBarScrollButton:disabled {
    UpIcon: url(:/assets/dark/tabbar/pageup_insensitive.svg);
    DownIcon: url(:/assets/dark/tabbar/pagedown_insensitive.svg);
}

TabBarScrollButton:pressed {
    UpIcon: url(:/assets/dark/tabbar/pageup_press.svg);
    DownIcon: url(:/assets/dark/tabbar/pagedown_press.svg);
}

TabBarScrollButton:hover {
    UpIcon: url(:/assets/dark/tabbar/pageup_hover.svg);
    DownIcon: url(:/assets/dark/tabbar/pagedown_hover.svg);
}

TabBarAddButton {
    BackgroundBrush: transparent;
    Icon: url(:/assets/dark/tabbar/add_normal.svg);
}

TabBarAddButton:pressed {
    Icon: url(:/assets/dark/tabbar/add_press.svg);
}

TabBarAddButton:hover {
    Icon: url(:/assets/dark/tabbar/add_hover.svg);
}

TabBarTab {
    TextColor: "#c0c0c0";
    BackgroundBrush: transparent;
    BorderBrush: rgba(0,0,0,0.1);
    ActiveColor: "#2ca7f8";
    CloseIcon: url(:/assets/dark/tabbar/tab_close_normal.svg);
}

TabBarTab:disabled {
    TextColor: "#949494";
}

TabBarTab:pressed {
    CloseIcon: url(:/assets/dark/tabbar/tab_close_press.svg);
}

TabBarTab:hover {
    BackgroundBrush: rgba(255, 255, 255, 0.05);
    CloseIcon: url(:/assets/dark/tabbar/tab_close_hover.svg);
}
